@@ -102,7 +102,7 @@ module Bundler
       end
 
       def identifier
-        uri_with_specifiers([humanized_ref, cached_revision, glob_for_display])
+        uri_with_specifiers([humanized_ref, locked_revision, glob_for_display])
       end
 
       def uri_with_specifiers(specifiers)
@@ -176,10 +176,10 @@ module Bundler
             "#{current_branch} but Gemfile specifies #{branch}"
         end
 
-        changed = cached_revision && cached_revision != revision
+        changed = locked_revision && locked_revision != revision
 
-        if !Bundler.settings[:disable_local_revision_check] && changed && !@unlocked && !git_proxy.contains?(cached_revision)
-          raise GitError, "The Gemfile lock is pointing to revision #{shortref_for_display(cached_revision)} " \
+        if !Bundler.settings[:disable_local_revision_check] && changed && !@unlocked && !git_proxy.contains?(locked_revision)
+          raise GitError, "The Gemfile lock is pointing to revision #{shortref_for_display(locked_revision)} " \
             "but the current branch in your local override for #{name} does not contain such commit. " \
             "Please make sure your branch is up to date."
         end
@@ -188,12 +188,10 @@ module Bundler
       end
 
       def specs(*)
-        set_up_app_cache!(app_cache_path) if use_app_cache?
+        set_cache_path!(app_cache_path) if use_app_cache?
 
         if requires_checkout? && !@copied
-          FileUtils.rm_rf(app_cache_path) if use_app_cache? && git_proxy.not_a_repository?
-
-          fetch
+          fetch unless use_app_cache?
           checkout
         end
 
@@ -216,17 +214,16 @@ module Bundler
         requires_checkout? ? spec.post_install_message : nil
       end
 
+      def migrate_cache(custom_path = nil, local: false)
+        if local
+          cache_to(custom_path, try_migrate: false)
+        else
+          cache_to(custom_path, try_migrate: true)
+        end
+      end
+
       def cache(spec, custom_path = nil)
-        app_cache_path = app_cache_path(custom_path)
-        return unless Bundler.feature_flag.cache_all?
-        return if install_path == app_cache_path
-        return if cache_path == app_cache_path
-        cached!
-        FileUtils.rm_rf(app_cache_path)
-        git_proxy.checkout if requires_checkout?
-        FileUtils.cp_r("#{cache_path}/.", app_cache_path)
-        FileUtils.touch(app_cache_path.join(".bundlecache"))
-        FileUtils.rm_rf(Dir.glob(app_cache_path.join("hooks/*.sample")))
+        cache_to(custom_path, try_migrate: false)
       end
 
       def load_spec_files
@@ -249,7 +246,7 @@ module Bundler
       end
 
       def app_cache_dirname
-        "#{base_name}-#{shortref_for_path(cached_revision || revision)}"
+        "#{base_name}-#{shortref_for_path(locked_revision || revision)}"
       end
 
       def revision
@@ -270,9 +267,38 @@ module Bundler
 
       private
 
+      def cache_to(custom_path, try_migrate: false)
+        return unless Bundler.feature_flag.cache_all?
+
+        app_cache_path = app_cache_path(custom_path)
+
+        migrate = try_migrate ? bare_repo?(app_cache_path) : false
+
+        set_cache_path!(nil) if migrate
+
+        return if cache_path == app_cache_path
+
+        cached!
+        FileUtils.rm_rf(app_cache_path)
+        git_proxy.checkout if migrate || requires_checkout?
+        git_proxy.copy_to(app_cache_path, @submodules)
+      end
+
       def checkout
         Bundler.ui.debug "  * Checking out revision: #{ref}"
-        git_proxy.copy_to(install_path, submodules)
+        if use_app_cache? && !bare_repo?(app_cache_path)
+          SharedHelpers.filesystem_access(install_path.dirname) do |p|
+            FileUtils.mkdir_p(p)
+          end
+          FileUtils.cp_r("#{app_cache_path}/.", install_path)
+        else
+          if use_app_cache? && bare_repo?(app_cache_path)
+            Bundler.ui.warn "Installing from cache in old \"bare repository\" format for compatibility. " \
+                            "Please run `bundle cache` and commit the updated cache to migrate to the new format and get rid of this warning."
+          end
+
+          git_proxy.copy_to(install_path, submodules)
+        end
         serialize_gemspecs_in(install_path)
         @copied = true
       end
@@ -320,13 +346,8 @@ module Bundler
         @install_path = path
       end
 
-      def set_up_app_cache!(path)
-        FileUtils.mkdir_p(path.join("refs"))
-        set_cache_path!(path)
-      end
-
       def has_app_cache?
-        cached_revision && super
+        locked_revision && super
       end
 
       def use_app_cache?
@@ -334,11 +355,11 @@ module Bundler
       end
 
       def requires_checkout?
-        allow_git_ops? && !local? && !cached_revision_checked_out?
+        allow_git_ops? && !local? && !locked_revision_checked_out?
       end
 
-      def cached_revision_checked_out?
-        cached_revision && cached_revision == revision && install_path.exist?
+      def locked_revision_checked_out?
+        locked_revision && locked_revision == revision && install_path.exist?
       end
 
       def base_name
@@ -375,7 +396,7 @@ module Bundler
         Bundler::Digest.sha1(input)
       end
 
-      def cached_revision
+      def locked_revision
         options["revision"]
       end
 
@@ -384,7 +405,7 @@ module Bundler
       end
 
       def git_proxy
-        @git_proxy ||= GitProxy.new(cache_path, uri, options, cached_revision, self)
+        @git_proxy ||= GitProxy.new(cache_path, uri, options, locked_revision, self)
       end
 
       def fetch
@@ -417,6 +438,10 @@ module Bundler
       def override_for(path)
         Bundler.settings.local_overrides.key(path)
       end
+
+      def bare_repo?(path)
+        File.exist?(path.join("objects")) && File.exist?(path.join("HEAD"))
+      end
     end
   end
 end
