@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.4.") or
-    raise "ruby lib version (3.4.1) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.4.2) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.4/riscv64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "4"
-  CONFIG["TEENY"] = "1"
-  CONFIG["PATCHLEVEL"] = "0"
-  CONFIG["INSTALL"] = '/local/pobj/ruby-3.4.1/bin/install -c'
+  CONFIG["TEENY"] = "2"
+  CONFIG["PATCHLEVEL"] = "28"
+  CONFIG["INSTALL"] = '/local/pobj/ruby-3.4.2/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)34"
@@ -111,7 +111,7 @@ module RbConfig
   CONFIG["CARGO_BUILD_ARGS"] = ""
   CONFIG["YJIT_SUPPORT"] = "no"
   CONFIG["CARGO"] = ""
-  CONFIG["RUSTC"] = "rustc"
+  CONFIG["RUSTC"] = "no"
   CONFIG["INSTALL_STATIC_LIBRARY"] = "no"
   CONFIG["modular_gc_dir"] = ""
   CONFIG["EXECUTABLE_EXTS"] = ""
@@ -162,16 +162,16 @@ module RbConfig
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/local/pobj/ruby-3.4.1/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/local/pobj/ruby-3.4.1/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/local/pobj/ruby-3.4.1/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/local/pobj/ruby-3.4.2/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/local/pobj/ruby-3.4.2/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/local/pobj/ruby-3.4.2/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
   CONFIG["WINDRES"] = ""
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 16.0.6\nTarget: riscv64-unknown-openbsd7.6\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 16.0.6\nTarget: riscv64-unknown-openbsd7.7\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["CSRCFLAG"] = ""
   CONFIG["COUTFLAG"] = "-o "
@@ -192,13 +192,13 @@ module RbConfig
   CONFIG["NM"] = "nm"
   CONFIG["LD"] = "ld"
   CONFIG["CXX"] = "c++"
-  CONFIG["AS"] = "gas"
+  CONFIG["AS"] = ""
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc"
   CONFIG["wasmoptflags"] = ""
   CONFIG["WASMOPT"] = ""
-  CONFIG["config_target"] = "riscv64-unknown-openbsd7.6"
-  CONFIG["target_os"] = "openbsd7.6"
+  CONFIG["config_target"] = "riscv64-unknown-openbsd7.7"
+  CONFIG["target_os"] = "openbsd7.7"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "riscv64"
   CONFIG["target"] = "$(target_cpu)-$(target_vendor)-$(target_os)"
@@ -206,16 +206,16 @@ module RbConfig
   CONFIG["host_vendor"] = "$(target_vendor)"
   CONFIG["host_cpu"] = "$(target_cpu)"
   CONFIG["host"] = "$(target)"
-  CONFIG["build_os"] = "openbsd7.6"
+  CONFIG["build_os"] = "openbsd7.7"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "riscv64"
-  CONFIG["build"] = "riscv64-unknown-openbsd7.6"
+  CONFIG["build"] = "riscv64-unknown-openbsd7.7"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
-  CONFIG["HAVE_GIT"] = "yes"
+  CONFIG["HAVE_GIT"] = "no"
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
